/*
 * Copyright (c) 2005 - 2007, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.oracle;

import java.sql.SQLException;
import java.util.Properties;
import javax.resource.ResourceException;
import oracle.jdbc.pool.OracleDataSource;
import org.tranql.connector.jdbc.AbstractLocalDataSourceMCF;

/**
 * ManagedConnectionFactory for connecting to a Oracle database with local (non-XA) transactions.
 *
 * @version $Revision: 711 $ $Date: 2010-04-27 17:11:13 -0700 (Tue, 27 Apr 2010) $
 */
public class LocalMCF extends AbstractLocalDataSourceMCF<OracleDataSource> {
    private String password;

    /**
     * Default constructor for an Oracle Local DataSource.
     *
     * @throws SQLException if there was a problem instantiating an OracleDataSource
     */
    public LocalMCF() throws SQLException {
        super(new OracleDataSource(), new OracleExceptionSorter(), true);
    }

    public void setDatabaseName(String dbname) {
        dataSource.setDatabaseName(dbname);
    }

    public String getDatabaseName() {
        return dataSource.getDatabaseName();
    }

    public void setServiceName(String serviceName) {
        dataSource.setServiceName(serviceName);
    }

    public String getServiceName() {
        return dataSource.getServiceName();
    }

    public void setDataSourceName(String dsname) {
        dataSource.setDataSourceName(dsname);
    }

    public String getDataSourceName() {
        return dataSource.getDataSourceName();
    }

    public void setDescription(String desc) {
        dataSource.setDescription(desc);
    }

    public String getDescription() {
        return dataSource.getDescription();
    }

    public void setNetworkProtocol(String np) {
        dataSource.setNetworkProtocol(np);
    }

    public String getNetworkProtocol() {
        return dataSource.getNetworkProtocol();
    }

    public void setPassword(String pwd) {
        this.password = pwd;
        dataSource.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public void setPortNumber(Integer pn) {
        dataSource.setPortNumber(pn == null ? 0 : pn.intValue());
    }

    public Integer getPortNumber() {
        return new Integer(dataSource.getPortNumber());
    }

    public void setServerName(String sn) {
        dataSource.setServerName(sn);
    }

    public String getServerName() {
        return dataSource.getServerName();
    }

    public void setUserName(String user) {
        dataSource.setUser(user);
    }

    public String getUserName() {
        return dataSource.getUser();
    }

    public String getDriverType() {
        return dataSource.getDriverType();
    }

    public void setDriverType(String s) {
        dataSource.setDriverType(s);
    }

    public String getTNSEntryName() {
        return dataSource.getTNSEntryName();
    }

    public void setTNSEntryName(String s) {
        dataSource.setTNSEntryName(s);
    }

    public String getURL() {
        try {
            return dataSource.getURL();
        } catch (SQLException e) {
            return "<unknown location>";
        }
    }

    public void setURL(String s) {
        dataSource.setURL(s);
    }

    public String getConnectionProperties() throws SQLException {
        return dataSource.getConnectionProperties().toString();
    }
    
    public void setConnectionProperties(String s) throws SQLException {
        // Should look something like the below, with any number of spaces around the "$" or "=" -
    	// "oracle.net.ssl_cipher_suites=(SSL_DH_anon_WITH_3DES_EDE_CBC_SHA,SSL_DH_anon_WITH_RC4_128_MD5)$MaxLimit=10"
    	// "oracle.net.ssl_cipher_suites=(SSL_DH_anon_WITH_3DES_EDE_CBC_SHA,SSL_DH_anon_WITH_RC4_128_MD5) $ MaxLimit=10"
    	if (s == null) {
    		return;
    	}
        Properties props = new Properties();
        String []args = s.trim().split("[$]");
        for (String arg : args) {
            String[] prop = arg.trim().split("[ =]+");
            if (prop.length == 2) {
                props.setProperty(prop[0], prop[1]);
            } 
            else {
                try {
                	if (getLogWriter() != null){
	                    getLogWriter().println("setConnectionProperties(" + s + ")");
	                    getLogWriter().println("Found invalid Oracle ConnectionProperties(" + prop.toString() + ")");
                	}
                } catch (ResourceException e) {
                    throw new SQLException(e.getMessage());
                }
            }
        }

        // Setup Connection Properties
        dataSource.setConnectionProperties(props);
    }

    public Integer getMaxStatements() throws SQLException {
        return new Integer(dataSource.getMaxStatements());
    }

    public void setMaxStatements(Integer ms) throws SQLException {
        int i = ms == null ? 0 : ms;
        dataSource.setMaxStatements(i);
        dataSource.setImplicitCachingEnabled(i != 0);
    }

    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     *
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof LocalMCF) {
            OracleDataSource other = ((LocalMCF) obj).dataSource;
            return equals(dataSource.getServerName(), other.getServerName()) &&
                    dataSource.getPortNumber() == other.getPortNumber() &&
                    equals(dataSource.getDatabaseName(), other.getDatabaseName());
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        return hashCode(dataSource.getServerName()) ^ dataSource.getPortNumber() ^ hashCode(dataSource.getDatabaseName());
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
    	try{
    		return "LocalMCF[" + getURL() + ";"+getConnectionProperties()+"]";	
    	} catch (Exception e) {
    		return "LocalMCF[" + getURL() + "]";
    	}
        
    }
}
