/*
 * Copyright (c) 2005 - 2009, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.oracle;

import java.sql.SQLException;
import java.util.Properties;

import javax.resource.ResourceException;
import oracle.jdbc.pool.OracleDataSource;
import org.tranql.connector.jdbc.AbstractLocalDataSourceMCF;

/**
 * ManagedConnectionFactory for connecting to a Oracle RAC enabled database with local (non-XA) transactions.
 *
 * @version $Revision: 663 $ $Date: 2009-09-24 23:48:55 -0700 (Thu, 24 Sep 2009) $
 */
public class LocalRACMCF extends AbstractLocalDataSourceMCF<OracleDataSource> {
    private static final long serialVersionUID = 8672506697045238199L;
    private String password;

    /**
     * Default constructor for an Oracle RAC Local DataSource.
     *
     * @throws SQLException if there was a problem instantiating an OracleDataSource
     */
    public LocalRACMCF() throws SQLException {
        super(new OracleDataSource(), new OracleRACExceptionSorter(), true);
        // the following property has to be set for the ONS.jar, even though its not used in thin mode
        System.setProperty("oracle.ons.oraclehome", ".");
    }

    public String getDatabaseName() {
        return dataSource.getDatabaseName();
    }

    public String getServiceName() {
        return dataSource.getServiceName();
    }

    public void setDataSourceName(String dsname) {
        dataSource.setDataSourceName(dsname);
    }

    public String getDataSourceName() {
        return dataSource.getDataSourceName();
    }

    public String getDescription() {
        return dataSource.getDescription();
    }

    public String getNetworkProtocol() {
        return dataSource.getNetworkProtocol();
    }

    public void setPassword(String pwd) {
        this.password = pwd;
        dataSource.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public Integer getPortNumber() {
        return dataSource.getPortNumber();
    }

    public String getServerName() {
        return dataSource.getServerName();
    }

    public void setUserName(String user) {
        dataSource.setUser(user);
    }

    public String getUserName() {
        return dataSource.getUser();
    }

    public String getDriverType() {
        return dataSource.getDriverType();
    }

    public String getTNSEntryName() {
        return dataSource.getTNSEntryName();
    }

    public String getURL() {
        try {
            return dataSource.getURL();
        } catch (SQLException e) {
            return "<unknown location>";
        }
    }
    
    public void setURL(String s) {
        dataSource.setURL(s);
    }

    public Integer getMaxStatements() throws SQLException {
        return dataSource.getMaxStatements();
    }

    public void setMaxStatements(Integer ms) throws SQLException {
        int i = ms == null ? 0 : ms;
        dataSource.setMaxStatements(i);
        dataSource.setImplicitCachingEnabled(i != 0);
    }

    // extra 3 config-property values for Oracle RAC
    public String getConnectionCacheName() throws SQLException {
        return dataSource.getConnectionCacheName();
    }
    
    public void setConnectionCacheName(String s) throws SQLException {
	    // the following have to be set together in this exact order
	    dataSource.setConnectionCachingEnabled(true);
        dataSource.setFastConnectionFailoverEnabled(true);
        dataSource.setConnectionCacheName(s);
    }
    
    public String getONSConfiguration() throws SQLException {
        return dataSource.getONSConfiguration();
    }
    
    public void setONSConfiguration(String s) throws SQLException {
        dataSource.setONSConfiguration(s);
    }
    
    public Properties getConnectionCacheProperties() throws SQLException {
        return dataSource.getConnectionCacheProperties();
    }
    
    public void setConnectionCacheProperties(String s) throws SQLException {
        // Should look something like the below, with any number of spaces around the "," or "=" -
        //   "InitialLimit=5,MinLimit=5,MaxLimit=10"
        //   "InitialLimit=5, MinLimit=5, MaxLimit=10"
        //   "InitialLimit = 5 , MinLimit = 5 , MaxLimit = 10"
        Properties props = new Properties();
        String []args = s.trim().split("[,]");
        for (String arg : args) {
            String[] prop = arg.trim().split("[ =]+");
            if (prop.length == 2) {
                props.setProperty(prop[0], prop[1]);
            } else {
                try {
                    getLogWriter().println("setConnectionCacheProperties(" + s + ")");
                    getLogWriter().println("Found invalid Oracle ConnectionCacheProperties(" + prop.toString() + ")");
                } catch (ResourceException e) {
                    throw new SQLException(e.getMessage());
                }
            }
        }

        // Setup ConnectionCaching Properties
        dataSource.setConnectionCacheProperties(props);
    }
        
    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     *
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof LocalRACMCF) {
            try {
                OracleDataSource other = ((LocalRACMCF) obj).dataSource;
                return equals(dataSource.getURL(), other.getURL()) &&
                       equals(dataSource.getConnectionCacheName(), other.getConnectionCacheName());
            } catch (SQLException e) {
                return false;
            }
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        try {
            return hashCode(dataSource.getURL()) ^ hashCode(dataSource.getConnectionCacheName());
        } catch (SQLException e) {
            return 0;
        }
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
        return "LocalRACMCF[" + getURL() + "]";
    }
}
